﻿#pragma once

#include "DirectXBase.h"

ref class ZoomMapRenderer sealed : public DirectXBase
{
private:
    static const int BITRES = 29;

    int integerLongitude;
    int integerLatitude;
    double fractionalZoomLevel;
    std::wstring uriTemplate;
    int minZoomLevel;
    int maxZoomLevel;
    int zoomLevelCount;
    int horzTileCount;
    int vertTileCount;
	bool m_renderNeeded;

    ID2D1Bitmap ** pBitmapTileArray;
    D2D1_POINT_2U * pPixelOffsetArray;

public:
	ZoomMapRenderer();
    virtual ~ZoomMapRenderer();

public:
	// DirectXBase methods.
    virtual void CreateDeviceIndependentResources() override;
    virtual void CreateDeviceResources() override;
    virtual void CreateWindowSizeDependentResources() override;
    virtual void Render() override;

    // Method for updating time-dependent objects.
    void Update(float timeTotal, float timeDelta);

    // Methods not in template called from DirextXPage
    bool NeedsRender();
    void SetGeolocation(double longitude, double latitude);
    void SetZoomLevel(double zoomLevel);
    void SetViewParams(Platform::String^ uriTemplate, int minZoomLevel, int maxZoomLevel);

private:
    // Methods not in template
    void DownloadAllMapTiles();
    Concurrency::task<ID2D1Bitmap*> LoadBitmap(Windows::Storage::Streams::IRandomAccessStreamReference^ streamRef, 
                                                Platform::String^ saveFilename);
    void SaveBitmapFile(Windows::Storage::Streams::IRandomAccessStream^ stream, Platform::String^ filename);
    void DrawTileArray(int zoomIndex, float scale, float opacity);
    std::wstring ToQuadKey(int longitude, int latitude, int level);
};
